# `prostore-axios-react`

Адаптер для
[prostore](https://gitlab.com/proscom/prostore/-/tree/master/packages%2Fprostore)
, связывающий
[prostore-axios](https://gitlab.com/proscom/prostore/-/tree/master/packages%2Fprostore-axios)
и
[prostore-react](https://gitlab.com/proscom/prostore/-/tree/master/packages%2Fprostore-react).

Содержит две полезных фичи:

- `AxiosClientsContext` - прокидывание клиента Apollo через контекст
- `useAxiosQuery` - хук, который
  динамически создают AxiosQueryStore в соответствии с переданными
  параметрами и подписывает на него компонент.

## AxiosClientsContext

Позволяет прокидывать клиент axios через контекст реакта.
Таким образом реализуется принцип Dependency Injection, и
снижается связанность кода.

```jsx harmony
// index.js
import axios from 'axios';
import {
  AxiosClientsContext,
  AxiosClientsManager
} from '@proscom/prostore-apollo-react';
import { App } from './App';

const defaultClient = axios.create();
const specificClient = axios.create({});

const axiosContext = new AxiosClientsManager({
  default: axiosClient,
  specific: axiosClient2
});

const rootElement = document.getElementById('root');

ReactDOM.render(
  <AxiosClientsContext.Provider value={axiosContext}>
    <App />
  </AxiosClientsContext.Provider>,
  rootElement
);
```

```jsx harmony
// App.js
function App() {
  const defaultClient = useContextAxiosClient();
  const specificClient = useContextAxiosClient('specific');

  // defaultClient === axiosClient
  // specificClient === axiosClient2

  // ...
}
```

## `useAxiosQuery`

Этот хук динамически создает стор AxiosQueryStore
и подписывают компонент на этот стор с помощью
[`useRequestStore`](https://gitlab.com/proscom/prostore/-/tree/master/packages%2Fprostore-react#useRequestStore)
.
Параметры, переданные в queryOptions, передаются в конструктор `AxiosQueryStore`.

```tsx
// Задаем запрос
const query: AxiosRequestConfig = {
  url: '/api/user',
  method: 'get'
};

// Это по-сути аргумент конструктора AxiosQueryStore
// должен быть за пределами компонента (или в useMemo),
// чтобы сохранять ссылочную постоянность, т.к. стор
// будет пересоздаваться при каждом изменении queryOptions
const queryOptions = {
  query,
  mapData: (result) => result.data,
  skipQuery: skipIfNull(null)
};

function UserInfo({ id }) {
  const userQuery = useAxiosQuery({
    queryOptions,
    // Сравниваются глубоко с помощью lodash.isEqual
    // если меняются, то запрос сам перевыполняется.
    // Можно перезаписать любую часть AxiosRequestConfig
    variables: { params: { id } },
    // Можно передать конкретный клиент Apollo, либо его ключ,
    // тогда он будет подключен из контекста
    client,
    // Дополнительные опции передаются в AxiosQueryStore.loadData
    options
  });
  const userData = userQuery.state.data;

  // С userQuery можно работать точно так же, как
  // с результатом вызова хука useRequestStore

  if (userQuery.check.spinner) {
    return <Spinner />;
  } else if (userQuery.check.error) {
    return <ErrorMessage error={userQuery.state.error} />;
  }

  return <div>{userData.name}</div>;
}
```

Так как `useAxiosQuery` создают
стор внутри компонента, то при ремаунте компонента стор будет
создан заново. Это приводит к повторным запросам на бекенд,
поэтому рекомендуется использовать какой-нибудь кеш, например
[`axios-cache-adapter`](https://www.npmjs.com/package/axios-cache-adapter).

## Примеры использования

### Создания axios-клиента с автоповтором запросов при ошибке сети и кешем

```typescript
import axios from 'axios';
import axiosRetry from 'axios-retry';
import { setupCache } from 'axios-cache-adapter';

const clientCache = setupCache({
  // Время жизни в кеше - 10 минут
  maxAge: 10 * 60 * 1000,
  // Задаем лимит количества сохраненных записей в кеше, чтобы вся память не съедалась
  limit: 100 as any, // Тут проблема с типами в библиотеке
  exclude: {
    // Кешируем запросы вместе с квери-параметрами
    query: false
  }
});

export const defaultClient = axios.create({
  // Используем кеширующий адаптер
  adapter: clientCache.adapter
});

// Повторяем запросы при ошибке сети до тех пор, пока она не исчезнет
axiosRetry(defaultClient, {
  // Бесконечное количество повторов
  retries: Infinity,
  // Повторный запрос через 5 секунд
  retryDelay: () => 5000
});
```

При использовании кеша может потребоваться удалить какое-нибудь значение из кеша.
Для этого можно использовать следующий код:

```typescript
import { serializeQuery } from 'axios-cache-adapter';

function clearQueryFromCache(url) {
  clientCache.store.iterate((value, key) => {
    if (key.startsWith(url)) {
      clientCache.store.removeItem(key);
    }
  });
}

clearQueryFromCache('/api/user');
// todo добавить поддержку query-параметров в clearQueryFromCache
```

Также можно вызвать запрос с инвалидацией кеша, передав `ignoreCache: true` в `options`:

```tsx
function UserInfo({ id }) {
  const userQuery = useAxiosQuery({
    /* ... */
  });

  const reloadData = () => {
    userQuery.load({ ignoreCache: true });
  };
}
```

Можно совместить инвалидацию кеша с [шиной событий](https://gitlab.com/proscom/prostore/-/tree/master/packages/prostore#шина-событий).
Для разрешение наибольшего количества ситуаций рекомендуется и сбрасывать кеш вручную, и вызывать запрос заново:

```tsx
const EVENT_USER_RELOAD = 'EVENT_USER_RELOAD';

storeEvents$.subscribe((event) => {
  if (event.type === EVENT_USER_RELOAD) {
    clearQueryFromCache('/api/user');
  }
});

function UserInfo({ id }) {
  const userQuery = useAxiosQuery({ params: { id } /* ... */ });

  const reloadData = () => {
    userQuery.load({ ignoreCache: true });
  };

  useObservable(storeEvents$, (event) => {
    if (event.type === EVENT_USER_RELOAD && id === event.id) {
      reloadData();
    }
  });

  return <div>...</div>;
}

function UserReload({ id }) {
  const handleClick = () => {
    storeEvents$.next({ type: EVENT_USER_RELOAD, id });
  };
  return <button onClick={handleClick}>Reload</button>;
}
```
