import { createContext, useContext } from 'react';
import {AxiosInstance} from 'axios';

export type IAxiosClients = {
  [name: string]: AxiosInstance;
};

/**
 * React context to hold AxiosClientManager instance
 */
export const AxiosClientsContext = createContext<AxiosClientsManager>(null);

/**
 * Used as the IoC container to inject axios client instances into components
 * based on their names
 */
export class AxiosClientsManager {
  constructor(public clients: IAxiosClients) {}

  getClient(key: string = 'default') {
    return this.clients[key];
  }
}

/**
 * Injects axios client from the context based on the provided name or
 * uses the provided instance. If called without arguments, uses the default
 * instance
 *
 * @param clientOrName - name of the axios client to be injected, or instance to be used
 */
export function useContextAxiosClient(
  clientOrName: string | AxiosInstance
): AxiosInstance {
  const clientsContext = useContext(AxiosClientsContext);

  if (!clientOrName) {
    return clientsContext.getClient();
  } else if (typeof clientOrName === 'string') {
    return clientsContext.getClient(clientOrName);
  } else {
    return clientOrName;
  }
}
